<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @subpackage [DB-related]
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 *
 */



/* TODO: the inter-connection between this and $db is not really clean, but this aim is to make the whole process cleaner.

/**
 *
 * NOTE: you're not supposed to deal with connectors any way
 *
 * @author Greg
 *
*/
class PHPDS_pdoConnector extends PHPDS_dependant implements iPHPDS_dbConnector
{
	/**
	 * @var string, DSN to use to connect to
	 */
	protected $dsn = '';
	
	/**
	 * @var string, username to use to connect with
	 */
	protected $user = '';
	
	/**
	 * @var string, password to use to connect with
	 */
	protected $password = '';
	
	/**
	 * @var php resource type,	the link for the mysql connection (as returned by mysql_connect)
	 */
	protected $link = null;

	/**
	 * @var php resource type,	the result resource of a query (as returned by mysql_query)
	 */
	protected $result;


	/**
	 * Clears the current connection (useful for example if we're fetching one row at a time and we give up before the end)
	 *
	 * @return nothing
	 */
	public function free()
	{
		if (!empty($this->result)) {
			$this->result = null;
		}
	}

	/**
	 * Connect to the database server (compatibility method)
	 *
	 * @date			20110810
	 * @version		1.0
	 * @author		greg
	 */
	public function connect()
	{
		// TODO: get our independance from db parameters and get our own
		// Does user want a persistant connection?
		/*if ($this->configuration['persistent_db_connection'] == true) {
			$this->link = mysql_pconnect($this->db->server, $this->db->dbUsername, $this->db->dbPassword);
		} else {
			$this->link = mysql_connect($this->db->server, $this->db->dbUsername, $this->db->dbPassword);
		}
		// Create database link.
		$ok = mysql_select_db($this->db->dbName, $this->link);
		// Display error on link.
		if (empty($ok)) 
			throw new PHPDS_databaseException(mysql_error());
		elseif (!empty($this->configuration['database_charset']))
			mysql_set_charset($this->configuration['database_charset']);*/
		
		if (empty($this->link)) {
			$this->link = new PDO($this->dsn, $this->user, $this->password);
			if (!is_a($this->link, 'PDO')) {
				throw new PHPDS_databaseException('Unable to create new PDO link ('.$this->dsn.')');
			}
		}
	}

	/**
	 * Actually send the query to MySQL (through $db)
	 *
	 * @date 20110810
	 * @version 1.0
	 * @author greg
	 * @param	 $sql string, the actual sql query
	 * @return php resource, the resulting resource (or false is something bad happened)
	 */
	public function query($sql)
	{
		$template = $this->template;
		try {
			$this->connect();
			// Replace the DB prefix.
			$configuration = $this->configuration;
			$real_sql = preg_replace('/_db_/', $configuration['database_prefix'], $sql);
			// Run query.
			// TODO: this should NOT rely on template
			if (! isset($template->stopScript) && ! empty($real_sql)) {
				// Count Queries Used...
				$this->db->countQueries ++;
				$this->_log($real_sql);
				$this->result = $this->link->query($real_sql);// or $template->stopScript = $this->returnSqlError($real_sql);
				if ($this->result == false) {
					$error = $this->link->errorInfo();
					throw new PHPDS_databaseException($error[2], $error[1]);
				}
				return $this->result;
			} else {
				return false;
			}
			// TODO: check result validity for non-select requests
		} catch (Exception $e) {
			$msg = '<p>The MySQL database engine returned with an error (code '. $e->getCode() . ' - ' . $e->getMessage() . '</p>';
			throw new PHPDS_databaseException($msg, 0, $e);
		}
	}

	/**
	 * Protect a single string from possible hacker (i.e. escape possible harmfull chars)
	 *
	 * @date			20100216
	 * @version 1.0
	 * @author	greg
	 * @param	$param		string, the parameter to espace
	 * @return string, the escaped string
	 * @see includes/PHPDS_db_connector#protect()
	 */
	public function protect($param)
	{
		// TODO: find another way
		return mysql_real_escape_string($param);
	}

	/**
	 * Return the next line as an associative array
	 *
	 * @date			20100216
	 * @version 1.0
	 * @author	greg
	 * @return array, the resulting line (or false is nothing is found)
	 * @see includes/PHPDS_db_connector#fetch_assoc()
	 */
	public function fetchAssoc()
	{
		$this->connect();
		
		return (is_a($this->result, 'PDOStatement')) ? $this->result->fetch(PDO::FETCH_ASSOC) : false;
	}

	/**
	 * Move the internal pointer to the asked line
	 *
	 * @date			20100216
	 * @version 1.0
	 * @author	greg
	 * @param	$row_number		integer, the line number
	 * @return boolean, TRUE on success or FALSE on failure
	 * @see includes/PHPDS_db_connector#seek()
	 */
	public function seek($row_number)
	{
		throw new PHPDS_exception('NOT IMPLEMENTED');
	}

	/**
	 * Return the number of rows in the result of the query
	 *
	 * @date			20100216
	 * @version 1.0
	 * @author	greg
	 * @return integer, the number of rows
	 * @see includes/PHPDS_db_connector#numrows()
	 */
	public function numrows()
	{
		$this->connect();
		return $this->result->rowCount();
	}

	/**
	 * Return the number of affected rows in the result of the query
	 *
	 * @date 20101103
	 * @version 1.0
	 * @author	Jason
	 * @return integer, the number of affected rows
	 * @see includes/PHPDS_db_connector#affectedRows()
	 */
	public function affectedRows()
	{
		$this->connect();
		return $this->result->rowCount();
	}

	/**
	 * This method returns the last MySQL error as a string if there is any. It will also
	 * return the actual erroneous SQL statement if the display_sql_on_error property is
	 * set to true. This is very helpfull when debugging an SQL related problem.
	 *
	 * @param string The actual query string.
	 * @return string
	 * @version 1.0.1
	 * @date 20100329 prevent an exception if display_sql_on_error is not set
	 * @author Don Schoeman <titan@phpdevshell.org>
	 */
	public function returnSqlError ($query)
	{
		$this->connect();
		$error = $this->link->errorInfo();
		if (empty($this->displaySqlOnError) && ! empty($error[0])) {
			$result = $error[0] . ": " . $error[2] . ' ['.$error[1].' <br />' . $query;
		}
		return $result;
	}

	/**
	 * Debugging Instance.
	 *
	 * @return debug object
	 */
	public function debugInstance ($ignored = null)
	{
		return parent::debugInstance('db');
	}

	/**
	 * Simply returns last inserted id from database.
	 *
	 * @date 20100610 (greg) (v1.0.1) added $this->link
	 * @version 1.0.1
	 * @author jason
	 * @return int
	 */
	public function lastId () 
	{
		$this->connect();
		return $this->link->lastInsertId();
	}

	/**
	 * Will return a single row as a string depending on what column was selected.
	 *
	 * @date 17062010 (jason)
	 * @version 1.0
	 * @author jason
	 * @return string
	 */
	public function rowResults ($row = 0)
	{
		throw new PHPDS_exception('NOT IMPLEMENTED');
	}

	/**
	 * Start SQL transaction.
	 */
	public function startTransaction()
	{
		$this->connect();
		$this->link->beginTransaction();
	}

	/**
	 * Ends SQL transaction.
	 *
	 * @param <type> $commit
	 */
	public function endTransaction($commit = true)
	{
		$this->connect();
		if ($commit) {
			$this->link->commit();
		} else {
			$this->link->rollback();
		}
	}

}
