<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */
// Get PHPMailer.
require_once ('includes/legacy/smarty/Smarty.class.php');
/**
 * Contains methods to handle emails appropriate.
 * PHPMailer could also be used.
 * @deprecated
 * @author Jason Schoeman
 */
class view extends Smarty
{
	/**
	 * Compiling directory.
	 *
	 * @var string
	 */
	public $compile_dir = 'default';
	/**
	 * Cache directory.
	 *
	 * @var string
	 */
	public $cache_dir = 'default';
	/**
	 * Config directory.
	 *
	 * @var string
	 */
	public $config_dir = 'default';

	/**
	 * Construct required files.
	 *
	 */
	public function __construct()
	{
		global $configuration;

		if ($this->compile_dir == 'default') $this->compile_dir = $configuration['absolute_path'] . $configuration['compile_path'];
		if ($this->cache_dir == 'default') $this->cache_dir = $configuration['absolute_path'] . $configuration['cache_path'];
		if ($this->config_dir == 'default') $this->config_dir = $configuration['absolute_path'] . 'config';
	}

	/**
	 * Loads the default or custom template (tpl) file and prints it out.
	 * Enter the template file for appropriate script here.
	 *
	 * @param string The name of the template to be loaded
	 * @param string If another plugin is to be used in the directory.
	 */
	public function show ($load_template=false, $plugin=false)
	{
		// Get correct tpl file.
		$tpl_dir = $this->get_tpl($load_template, $plugin);
		if (empty($tpl_dir)) return false;
		// Execute template else just skip it.
		$this->display($tpl_dir);
	}

	/**
	 * Loads the default or custom template (tpl) file and returns results.
	 * Enter the template file for appropriate script here.
	 *
	 * @param string The name of the template to be loaded
	 * @param string If another plugin is to be used in the directory.
	 */
	public function dofetch ($load_template=false, $plugin=false)
	{
		// Get correct tpl file.
		$tpl_dir = $this->get_tpl($load_template, $plugin);
		if (empty($tpl_dir)) return false;
		// Execute template else just skip it.
		return $this->fetch($tpl_dir);
	}

	/**
	 * Gets the correct location of a tpl file.
	 *
	 * @param string $load_template
	 * @param string $plugin If another plugin is to be used in the directory.
	 */
	public function get_tpl ($load_template=false, $plugin=false)
	{
		global $configuration, $navigation, $errorHandler, $core, $template;

		// Menu link.
		if (empty($navigation->navigation[$configuration['m']]['extend'])) {
			$menu_link = $navigation->navigation[$configuration['m']]['menu_link'];
		} else {
			$menu_link = $navigation->navigation[$navigation->navigation[$configuration['m']]['extend']]['menu_link'];
			// Set plugin for this menu item.
			$plugin = $navigation->navigation[$navigation->navigation[$configuration['m']]['extend']]['plugin'];
		}
		// Do template engine.
		if (empty($plugin) || ! empty($navigation->navigation[$configuration['m']]['layout'])) {
			$plugin_folder = $configuration['absolute_path'] . $navigation->navigation[$configuration['m']]['plugin_folder'];
		} else {
			$plugin_folder = $configuration['absolute_path'] . 'plugins/' . $plugin . '/';
		}
		// Do we have a custom template file?
		if (empty($load_template) && ! empty($navigation->navigation[$configuration['m']]['layout'])) {
			$load_template = $navigation->navigation[$configuration['m']]['layout'];
		}
		// Check if we have a template file otherwise use default.
		if (empty($load_template)) {
			$tpl_dir = str_replace($menu_link, 'templates/' . str_replace('.php', '.tpl', $menu_link), $plugin_folder . $menu_link);
		} else {
			$link = strrchr($menu_link, '/');
			if (empty($link)) {
				$tpl_dir = $plugin_folder . 'templates/' . $load_template;
			} else {
				$link = str_replace($link, '', $menu_link);
				$tpl_dir = $plugin_folder . 'templates/' . $link . '/' . $load_template;
			}
		}
		// Log to firephp.
		$template->_log(___('Loading Template Layout : ') . $tpl_dir);
		// Return file location.
		if (file_exists($tpl_dir)) {
			return $tpl_dir;
		} else {
			return false;
		}
	}
}
?>