<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */
class RegistrationTokenAdmin extends PHPDS_controller
{

	/**
	 * Administration of regisrtation tokens
	 * @author Jason Schoeman [titan@phpdevshell.org], Ross Kuyper
	 * @since 06 July 2010
	 */
	public function execute()
	{

		// Define.
		$verify_registration_option0 = false;
		$verify_registration_option1 = false;
		// Header
		if (!empty($this->security->get['erg']) || !empty($this->security->post['save'])) {
			// Create header type.
			$this->template->heading(_('Edit Registration Token'));
			
		} else {
			// Create header type.
			$this->template->heading(_('Add Registration Token'));
			
			// Define unused variables.
			$edit['user_role_id'] = 0;
			$edit['user_group_id'] = 0;
			$edit['registration_option'] = '';
			$edit['token_id'] = '';
			$edit['token_name'] = '';
			$edit['available_tokens'] = 0;
		}
		// Post and save.
		if (!empty($this->security->get['erg']) || !empty($this->security->post['save']) || !empty($this->security->post['new'])) {
			// If item is in edit mode.
			if (!empty($this->security->get['erg'])) {
				$edit = $this->db->invokeQuery('PHPDS_getTokenQuery', $this->security->get['erg']);
			}
			// If item is saved.
			if (!empty($this->security->post['save']) || !empty($this->security->post['new'])) {
				// Capture submitted data.
				$edit = $this->security->post;
				if (empty($edit['token_id'])) $edit['token_id'] = 0;

				// Some minor error checking.
				if (empty($edit['token_name']) || empty($edit['user_role_id']) || empty($edit['user_group_id'])) {
					$this->template->warning(_('You missed fields required to save registration token.'));
					$error[1] = true;
				} else {
					// Does the record already exist.
					if ($this->db->doesRecordExist('_db_core_registration_tokens', 'token_name', "{$edit['token_name']}", 'token_id', "{$edit['token_id']}") == true) {
						$this->template->warning(sprintf(_('The registration token "%s" you specified already exists, please use a different name.'), $edit['token_name']));
						$error[2] = true;
					}
				}
				// If everything is ok we can now save.
				if (empty($error)) {

					$edit['token_id'] = $this->db->invokeQuery('PHPDS_saveRegistrationTokensQuery', $edit['token_id'], $edit['token_name'], $edit['user_role_id'], $edit['user_group_id'], $edit['token_key'], $edit['registration_option'], $edit['available_tokens']);

					// Set variable global for hooks.
					$this->template->global['token_id'] = $edit['token_id'];

					// Show item updated.
					$this->template->ok(sprintf(_('The registration token "%s" was saved as required.'), $edit['token_name']));
				}
			}
		}
		// Query available roles.
		$user_roles_option_move = $this->db->invokeQuery('PHPDS_tokensGetRolesQuery', $edit);

		// Query available groups.
		$user_groups_option_move = $this->db->invokeQuery('PHPDS_tokensGetGroupsQuery', $edit);

		// Save and new item.
		if (!empty($this->security->post['new'])) {
			$edit['token_id'] = 0;
			$edit['token_name'] = '';
			$edit['token_key'] = '';
		}

		// Generate random registration token.
		if (empty($edit['token_key']))
			$edit['token_key'] = $this->core->createRandomString(42);
		// Allow Token.
		($edit['registration_option'] == '1') ? $verify_registration_option0 = 'checked' : $verify_registration_option1 = 'checked';

		// Load views.
		$view = $this->factory('views');

		// Set Array.
		$view->set('e', $edit);

		// Set Values.
		$view->set('self_url', $this->navigation->selfUrl());
		$view->set('user_roles_option_move', $user_roles_option_move);
		$view->set('user_groups_option_move', $user_groups_option_move);
		$view->set('verify_registration_option0', $verify_registration_option0);
		$view->set('verify_registration_option1', $verify_registration_option1);

		// Output Template.
		$view->show();
	}
}

return 'RegistrationTokenAdmin';