<?php

/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

/**
 * Optimize Database - Fetch Tables
 * @author Jason Schoeman [titan@phpdevshell.org], Ross Kuyper, Contact: rosskuyper@gmail.com.
 *
 */
class PHPDS_fetchTablesToOptimizeQuery extends PHPDS_query
{
	protected $sql = "
		SHOW TABLES FROM
			%s
	";
}

/**
 * Optimize Database - Optimize Tables
 * @author Jason Schoeman [titan@phpdevshell.org], Ross Kuyper, Contact: rosskuyper@gmail.com.
 *
 */
class PHPDS_optimizeTablesQuery extends PHPDS_query
{
	protected $sql = "
		OPTIMIZE TABLE
			%s		
	";

	/**
	 * Initiate query invoke command.
	 * @param array
	 * @return array
	 */
	public function invoke($parameters = null)
	{
		// Define.
		$table_data = $parameters[0];
		$tables_ = '';
		$tables = '';
		$tables_to_optimize = '';

		// Fetch table data.
		foreach ($table_data as $table_name => $all_tables_array) {
			$tables = $table_name;
			$tables_ .= $tables . ',';
		}

		// Write DB string.
		$tables_to_optimize = rtrim($tables_, ',');

		// Query the database
		$optimize_results = parent::invoke($tables_to_optimize);

		// Print Results
		foreach ($optimize_results as $optimize_results_array) {
			$table = $optimize_results_array['Table'];
			$Op = $optimize_results_array['Op'];
			$msg_type = $optimize_results_array['Msg_type'];
			$msq_text = $optimize_results_array['Msg_text'];
			// This is used for template preview when debugging.
			$RESULTS[] = array(
				'table' => $table,
				'Op' => $Op,
				'msg_type' => $msg_type,
				'msq_text' => $msq_text
			);
		}
		if ($RESULTS) return $RESULTS;
		else return false;
	}
}