<?php
/**
 * PHPDevShell is a RAD Framework aimed at developing administrative applications.
 *
 * @package PHPDevShell
 * @link http://www.phpdevshell.org
 * @copyright Copyright (C) 2007 Jason Schoeman, All rights reserved.
 * @license GNU/LGPL, see readme/licensed_under_lgpl or http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * @author Jason Schoeman, Contact: titan [at] phpdevshell [dot] org.
 *
 * Copyright notice: See readme/notice
 * By using PHPDevShell you agree to notice and license, if you dont agree to this notice/license you are not allowed to use PHPDevShell.
 */

function mod_logo($url, $src, $alt, $title)
{
	return "
		<a href=\"{$url}\">
			<img src=\"{$src}\" class=\"logo\" alt=\"{$alt}\" title=\"{$title}\">
		</a>
			
	";		
}

function mod_logo_text($title)
{
	return '<h1>' . $title . '</h1>
		';
}

function mod_title($title, $appname)
{
	return "{$title} - {$appname}";
}

function mod_role($role, $name) 
{
	return "{$name} {$role}&#44;&nbsp;";
}

function mod_group($group, $name) 
{
	return "{$name} {$group}&#44;&nbsp;";
}

function mod_css_file_to_head($href, $media)
{
	return '<link rel="stylesheet" href="' . $href . '" media="' . $media . '" />
		';
}

function mod_js_file_to_head($src)
{
	return '<script type="text/javascript" src="' . $src . '"></script>
		';
}	

function mod_add_to_head($head)
{
	return "<!-- Dynamically Added to Head -->
			$head
	
			";
}

function mod_add_js_to_head($js)
{
	return '<script type="text/javascript">' . $js . '</script>
		';
}

function mod_add_css_to_head($css)
{
	return '<style>' . $css . '</style>
		';
}

function mod_iframe($src, $height, $width)
{
	$HTML = <<<HTML
		<iframe src="{$src}" name="iframe" height="{$height}" frameborder="0" scrolling="auto" width="{$width}" seamless></iframe>	
			
HTML;
	return $HTML;
}

function mod_result($colspan, $td_content)
{
	$HTML = <<<HTML
		<tr class="highlight">
			<td class="no_results" colspan="{$colspan}">
				{$td_content}
			</td>
		</tr>
		
HTML;
	return $HTML;
}

function mod_login_form($action, $username_label, $password_label, $redirect_page, $lost_password, $lost_password_text, $not_registered_yet, $not_registered_yet_text, $remember, $security, $login_label, $user_name)
{
	if (! empty($remember)) {
		$remember = '<label>' . $remember . '<input tabindex="3" type="checkbox" name="user_remember" value="remember"></label>';
	} else {
		$remember = '';
	}
	
	if (! empty($not_registered_yet)) {
		$not_registered_yet = '<a href="' . $not_registered_yet . '">' . $not_registered_yet_text . '</a>';
	} else {
		$not_registered_yet = '';
	}
	
	$HTML = <<<HTML
   		<form id="login" action="{$action}" method="post" class="validate">
				<div class="row">
						<div class="column grid_6">
								<p><label>{$username_label}<input tabindex="1" type="text" size="20" name="user_name" required="required" value="{$user_name}"></label></p>
								<p><label>{$password_label}<input tabindex="2" type="password" size="20" name="user_password" required="required"></label></p>
								{$redirect_page}
								<p>{$remember}</p>
								<p>
									<button type="submit" name="login"><span class="ui-icon ui-icon-key left"></span><span>{$login_label}</span></button>
									<a href="{$lost_password}">{$lost_password_text}</a>
									{$not_registered_yet}
								</p>
								<input type="hidden" name="login" value="login">
								{$security}
						</div>
				</div>
		</form>
		
HTML;
	return $HTML;
}

function mod_heading($heading_text)
{
	$HTML = <<<HTML
   		<div id="heading" class="ui-widget-header ui-corner-all">$heading_text</div>
			
HTML;
	return $HTML;
}

function mod_info($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all info">
			<span class="ui-icon ui-icon-help left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_scriptIcon($src, $title)
{
	$HTML = <<<HTML
		<img class="img" src="{$src}" title="{$title}" alt="{$title}" />
			
HTML;
	return $HTML;
}

function mod_scripthead($text)
{
	$HTML = <<<HTML
		<div class="ui-state-default ui-corner-all scripthead">
			<span class="ui-icon ui-icon-tag left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_error($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all error">
			<span class="ui-icon ui-icon-circle-close left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_warning($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all warning">
			<span class="ui-icon ui-icon-notice left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_critical($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all critical">
			<span class="ui-icon ui-icon-alert left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_notice($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all notice">
			<span class="ui-icon ui-icon-lightbulb left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_busy($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all busy">
			<span class="ui-icon ui-icon-flag left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_message($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all message">
			<span class="ui-icon ui-icon-comment left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_note($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all note">
			<span class="ui-icon ui-icon-pencil left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_ok($text)
{
	$HTML = <<<HTML
		<div class="ui-corner-all ok">
			<span class="ui-icon ui-icon-check left"></span>
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_icon($src, $class, $title)
{
	$HTML = <<<HTML
		<img src="{$src}" class="icon {$class}" title="{$title}" alt="{$title}" />
			
HTML;
	return $HTML;
}

function mod_tooltip($text)
{
	$HTML = <<<HTML
		<a href="#" title="{$text}" class="tooltip"><span class="ui-icon ui-icon-info"></span></a>
			
HTML;
	return $HTML;
}

function mod_no_results($text)
{
	$HTML = <<<HTML
		<div id="norecords" class="ui-state-disabled ui-corner-all">
			{$text}
		</div>
		
HTML;
	return $HTML;
}

function mod_results($first_page, $rw, $previous_page, $currentPage_, $total_pages_, $current_records_, $totalRows_, $next_page, $ff, $last_page)
{
	$HTML = <<<HTML
		<div id="pagination">
			<ul id="results" class="ui-widget">
				{$first_page}
				{$rw}
				{$previous_page}
				<li class="pages ui-state-default ui-corner-all"><span class="ui-icon ui-icon-clipboard left"></span>[{$currentPage_}/{$total_pages_}] - [{$current_records_}/{$totalRows_}]</li>
				{$next_page}
				{$ff}
				{$last_page}
			</ul>
		</div>
		
HTML;
	return $HTML;
}

function mod_search($action, $value, $class, $validate)
{
	$HTML = <<<HTML
		<form action="{$action}" method="post">
			<div id="searchForm">
				<span class="ui-icon ui-icon-search left"></span><input id="search_field" type="text" size="40" name="search_field" value="{$value}" class="{$class}">
				<input type="hidden" value="Filter" name="search">
				{$validate}
			</div>
		</form>
		
HTML;
	return $HTML;
}

function mod_info_mark($icon, $text)
{
	$HTML = <<<HTML
		<a href="#" class="toolpopup">
			{$icon}
			<span>
				{$text}
			</span>
		</a>
		
HTML;
	return $HTML;
}

function mod_loggedininfo($href, $text)
{
	return "<a href=\"{$href}\"><div id=\"logged-in\" class=\"loginlink\"><span></span>{$text}</div></a>
	";
}

function mod_logininfo($href, $inoutpage)
{
	return "<a href=\"{$href}\"><div id=\"logged-out\" class=\"loginlink\"><span></span>{$inoutpage}</div></a>
	";
}

function mod_pagination_order($order_url, $asc, $desc)
{
	$HTML = <<<HTML
		<div class="order">
			<a href="{$order_url}&order=asc"><span class="asc {$asc}"></span></a>
			<a href="{$order_url}&order=desc"><span class="desc {$desc}"></span></a>
		</div>
		
HTML;
	return $HTML;
}

function mod_pagination_th($th_, $sort_html)
{
	return "<th>{$th_}{$sort_html}</th>
			";
}

function mod_pagination_nav($url, $class)
{
	return "<li class=\"paginationicon\">
				<a href=\"{$url}\">
					<span class=\"{$class}\"></span>
				</a>
			</li>
			";
}

function mod_pagination_nav_empty($class)
{
	return "<li class=\"paginationicondisabled\">
				<span class=\"{$class}\"></span>
			</li>
			";
}

function mod_menu_a($mr, $class='')
{
	// Check if we have a place marker.
	if ($mr['menu_type'] == 6 || ($mr['hide'] != 0 && $mr['hide'] != 2)) {
		$noclick = 'onclick="return false;"';
		// Create URL.
		$url = "&#35;";
	} else {
		$noclick = '';
		// Last check if it is a link item that should be jumped to.
		if ($mr['menu_type'] == 5) {
			$url = $mr['menu_link'];
		} else {
			$url = $mr['href'];
		}
	}
	($mr['new_window'] == 1) ? $target = '_blank' : $target = '_self';
	
	/**
	 * Class types:
	 * nav-grand
	 * nav-parent
	 * child
	 */
	if (empty($class))
		$span = '';
	else 
		$span = "<span class=\"{$class}\"></span>
		";

	return "<a href=\"{$url}\" target=\"{$target}\" {$noclick}>
				{$span}{$mr['menu_name']}
			</a>
			";
}

function mod_menu_ul_parent($tree, $class)
{	
	/**
	 * Class types:
	 * navparent
	 * ulchild
	 * breadparent
	 */
	return "<ul class=\"{$class}\">
				{$tree}
			</ul>
			";
}

function mod_menu_ul_child($tree, $class)
{	
	/**
	 * Class types:
	 * navparent
	 * ulchild
	 * breadparent
	 */
	return "<ul class=\"{$class}\">
				{$tree}
			</ul>
			";
}

function mod_menu_li_parent($tree, $link, $class)
{	
	/**
	 * Class types:
	 * parent
	 * grandparent
	 * current
	 * inactive
	 */
	return "<li class=\"{$class}\">
				{$link}
					{$tree}
			</li>
			";
}

function mod_menu_li_child($link, $class)
{	
	/**
	 * Class types:
	 * current
	 * inactive
	 */
	return "<li class=\"{$class}\">
				{$link}
			</li>
			";
}

function mod_menu_li_jump($href, $name)
{	
	return "<li class=\"jump\">
				<a href=\"{$href}\">
					<span></span>{$name}
				</a>
			</li>
			";
}

function mod_menu_li_up($href, $name)
{	
	return "<li class=\"up\">
				<a href=\"{$href}\">
					<span></span>{$name}
				</a>
			</li>
			";
}

function mod_menu_li_home($href, $name, $jump_menu)
{	
	return "<li class=\"home\">
				<a href=\"{$href}\">
					<span></span>{$name}
				</a>
			</li>
			{$jump_menu}
			";
}

function mod_menu_redirect($url, $time)
{	
	return '<META HTTP-EQUIV="refresh" CONTENT="' . $time . '; URL=' . $url . '">
		';
}

function mod_security_token($token)
{
	return '<input type="hidden" name="token_validation" value="' . $token . '">
		';
}

function mod_search_token($search_token)
{
	return '<input type="hidden" name="search_token_validation" value="' . $search_token . '">
		';
}

function mod_debug($queries, $memory, $other='')
{
	return "<div id=\"queries\">
				(Queries Used : {$queries}) - (PHP Memory Used : {$memory} Mb)<br>{$other}
			</div>
				";	
}

function mod_widget($widget_url, $element_id, $text='', $options = '') {

		$JS = <<<JS

				$(document).ready(function() {
					$.ajax({
					  url: "{$widget_url}",
					  dataType: 'html',
					  {$options}
					  data: "widget=true",
					  beforeSend: function() {
						 $("#{$element_id}").append('<img id="loading_{$element_id}" src="themes/cloud/images/widget-loader.gif" title="{$text}" alt="{$text}" />');
					  },
					  complete: function(){
						 $("#loading_{$element_id}").hide()
					  },
					  success: function(html){
						$("#{$element_id}").append(html).hide().fadeIn('slow');
					  }
					});
				});
				
JS;
	return $JS;	
}

function mod_ajax($ajax_url, $element_id, $text='', $options = '') {

		$JS = <<<JS

				$(document).ready(function() {
					$.ajax({
					  url: "{$ajax_url}",
					  dataType: 'html',
					  {$options}
					  data: "ajax=true",
					  beforeSend: function() {
						 $("#{$element_id}").append('<img id="loading_{$element_id}" src="themes/cloud/images/ajax-loader.gif" title="{$text}" alt="{$text}" />');
					  },
					  complete: function(){
						 $("#loading_{$element_id}").hide()
					  },
					  success: function(html){
						$("#{$element_id}").append(html).hide().fadeIn('slow');
					  }
					});
				});
				
JS;
	return $JS;
}

function mod_lightbox($element_id, $options = '') {

		$JS = <<<JS
	
			$(document).ready(function() {
				$("#$element_id").fancybox({
					{$options}
					'ajax' : { data: "lightbox=true" }
				});
			});	
			
JS;
	return $JS;
}


